'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getChannels = getChannels;
exports.sendMsg = sendMsg;
exports.commOpenMessages = commOpenMessages;
exports.commCloseMessages = commCloseMessages;
exports.commMessages = commMessages;
exports.getCommId = getCommId;
exports.commIdFilter = commIdFilter;
exports.getCommTargetName = getCommTargetName;
exports.getMessageData = getMessageData;
exports.openComm = openComm;
exports.sendCommMessage = sendCommMessage;

var _uuid = require('uuid');

var uuid = _interopRequireWildcard(_uuid);

var _Rx = require('rxjs/Rx');

var _Rx2 = _interopRequireDefault(_Rx);

var _messaging = require('./messaging');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

/** TODO: Move to more generic location
 * Get an Observable for kernel channels.
 *
 * To get the current channel object, use `getChannels(store).first().toPromise()`.
 * This observable is designed to permit past, present, and future subscriptions.
 * For a POC, see http://jsbin.com/tujapaciwo/5/edit?js,console
 * @param  {Redux.store} store
 * @return {Observable} observable of the kernel channels object
 */
function getChannels(store) {
  // For a proof of concept, see the following JSBin:
  // http://jsbin.com/tujapaciwo/5/edit?js,console

  // Defer the initial value till the channels are subscribed to.  This will
  // make it so the action of a subscription will cause the current channels to
  // be emitted, regardless of whether or not they were set long before when
  // the subscriber subscribes.
  return _Rx2.default.Observable.defer(() => {
    let currentChannels;
    if (store.app) {
      currentChannels = store.app.channels;
    }
    return currentChannels;
  })

  // Listen to channel change events.
  .merge(_Rx2.default.Observable.from(store).pluck('app').pluck('channels'))

  // Don't emit null or undefined values, only emit changes, and remember the
  // last state for delayed subscriptions.
  .filter(Boolean).distinctUntilChanged().publishReplay(1) // Only remember the last state
  .refCount();
}

/** TODO: Move to more generic location
 * Send a message on the connected, or soon to be connected, kernel.
 * @param  {Redux.store} store
 * @param  {string} channel - name of the chanel to send the message on
 * @param  {object} msg - Jupyter message to send
 * @return {Promise} Promise that resolves when the message has been sent
 */
/**
 * Currently unused, leaving here for when the widgets are brought back in
 */

function sendMsg(store, channel, msg) {
  return getChannels(store).pluck(channel).first().toPromise().then(channelSubject => {
    const tempSubscription = channelSubject.subscribe(() => {});
    channelSubject.next(msg);
    tempSubscription.unsubscribe();
  });
}

/**
 * Get an Observable for comm related messages.
 * @param  {Redux.store} store
 * @return {Observable} Observable of comm related messages.
 */
function commRelatedMessages(store) {
  return getChannels(store).switchMap(channels => {
    if (!channels.iopub) {
      return _Rx2.default.Observable.empty();
    }
    return channels.iopub.filter(msg => msg && msg.header && msg.header.msg_type && msg.header.msg_type.slice(0, 5) === 'comm_');
  });
}

/**
 * Get an Observable for comm open messages.
 * @param  {Redux.store} store
 * @return {Observable} Observable of comm open messages.
 */
function commOpenMessages(store) {
  return commRelatedMessages(store).filter(msg => msg.header.msg_type === 'comm_open');
}

/**
 * Get an Observable for comm close messages.
 * @param  {Redux.store} store
 * @return {Observable} Observable of comm close messages.
 */
function commCloseMessages(store) {
  return commRelatedMessages(store).filter(msg => msg.header.msg_type === 'comm_close');
}

/**
 * Get an Observable for comm "msg" messages (normal communications).
 * @param  {Redux.store} store
 * @return {Observable} Observable of comm messages.
 */
function commMessages(store) {
  return commRelatedMessages(store).filter(msg => msg.header.msg_type === 'comm_msg');
}

/**
 * Get the comm id of a comm message.
 * @param  {object} msg
 * @return {string} comm id
 */
function getCommId(msg) {
  return msg.content.comm_id;
}

/**
 * Make a message filter for a specific comm
 * @param  {string} commId - comm id of messages to filter for
 * @return {function} predicate function (filter)
 */
function commIdFilter(commId) {
  return msg => getCommId(msg) === commId;
}

/**
 * Get the target name of a comm open message.
 * @param  {object} msg
 * @return {string} target name
 */
function getCommTargetName(msg) {
  return msg.content.target_name;
}

/**
 * Get the comm message data
 * @param  {object} msg - comm "msg" message
 * @return {object} message data
 */
function getMessageData(msg) {
  return msg.content.data;
}

/**
 * Opens a new comm.
 * @param  {Redux.store} store
 * @param  {string} commTargetName - backend target name of the comm to create
 * @param  {object} [data] - optional, associated comm open data
 * @param  {object} [originalHeader] - optional, parent header
 * @return {Promise<{commId, msgId}>} Promise for the new comm's id and the id
 *                                    of the msg sent to open the comm.
 */
function openComm(store, commTargetName, data, originalHeader) {
  const commId = uuid.v4();
  const openMsg = (0, _messaging.createMessage)('comm_open');
  openMsg.content = {
    comm_id: commId,
    target_name: commTargetName,
    data
  };
  openMsg.parent_header = originalHeader || openMsg.parent_header;

  return sendMsg(store, 'shell', openMsg).then(() => ({
    commId,
    msgId: openMsg.header.msg_id
  }));
}

/**
 * Creates and sends a comm message.
 * @param  {Redux.store} store
 * @param  {string} commId - id of the comm to send a message to
 * @param  {object} data - comm msg data
 * @param  {object} [originalHeader] - optional, parent header
 * @return {Promise<string>} Promise for the message id
 */
function sendCommMessage(store, commId, data, originalHeader) {
  const msg = (0, _messaging.createMessage)('comm_msg');
  msg.content = {
    comm_id: commId,
    data
  };
  msg.parent_header = originalHeader || msg.parent_header;

  return sendMsg(store, 'shell', msg).then(() => msg.header.msg_id);
}